package problemes.tsp;

import java.util.LinkedList;

public class CarteRoutiereRoumanie {
	
	public static enum Villes {Arad,Bucharest,Craiova,
		Dobreta,Fagaras,Oradea,Pitesti,Riminiscu,Sibiu};
		
	private static int[][] distance = 
	{
		{   0,  -1,  -1, 374,  -1, 146,  -1,  -1, 140 },
		{  -1,   0, 160,  -1, 211,  -1, 101,  -1,  -1 },
		{  -1, 160,   0, 120,  -1,  -1, 138, 146,  -1 },
		{ 374,  -1, 120,   0,  -1,  -1,  -1,  -1,  -1 },
		{  -1, 211,  -1,  -1,   0,  -1,  -1,  -1,  99 },
		{ 146,  -1,  -1,  -1,  -1,   0,  -1,  -1, 151 },
		{  -1, 101, 138,  -1,  -1,  -1,   0,  97,  -1 },
		{  -1,  -1, 146,  -1,  -1,  -1,  97,   0,  80 },
		{ 140,  -1,  -1,  -1,  99, 151,  -1,  80,   0 }
			
	};
	
	/* Distance la plus courte entre deux villes reliees par une route */
	public int distanceMinimum(){
		return 80;
	}
	
	/* Teste si nom correspond a une ville de la carte */
	public boolean checkVille(String nom){
		try {
			 Villes.valueOf(nom);
		} catch(java.lang.IllegalArgumentException e) {
                    System.out.println("Erreur : "+ nom + " n'est pas une ville de la carte actuelle");
			return false;
		}
		//System.out.println(Villes.valueOf(nom));
		return true;
	}
	
	
	/* Renvoie un entier positif correspondant à la longueur de la route
	 * Si une route existe entre les deux villes. 
	 * Renvoie -1 sinon 
	 *  */
	public int connexion(Villes va, Villes vb){
		return distance[va.ordinal()][vb.ordinal()];
	}
	
	
	public int connexion(String sa, String sb){
		int va = Villes.valueOf(sa).ordinal();
		int vb = Villes.valueOf(sb).ordinal();
		return distance[va][vb];
	}
	
	
	/* Retourne la liste des villes de la carte */
	public LinkedList<String> getListeVilles(){
		LinkedList<String> toRet = new LinkedList<String>(); 
		for (int i=0; i< Villes.values().length; i++)
			toRet.add(Villes.values()[i].toString());
		return toRet;
	}
	
	
	/* A partir d'une ville s, retourne la route la plus courte
	   qui passe par elle.  */
	public int routeLaPlusCourte(String s){
		int va = Villes.valueOf(s).ordinal();
		int min=Integer.MAX_VALUE;
		for (int i=0; i < distance.length; i++){
			if ((distance[i][va]<min) && ((distance[i][va]>0)))
				min =distance[i][va];
			
		} 
		return min;
	}
	
	
}
